#!/usr/bin/python3

# Copyright (C) 2012-2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

"""
Reads a configuration file for Homewatcher and resolves its parameterized parts before dumping it again.
"""

# Check that pyknx is present as soon as possible.
from homewatcher import ensurepyknx

from homewatcher.configuration import Configuration
import argparse
import sys
import logging
import os
from pyknx import logger

__doc__ = __doc__.format(scriptname=os.path.basename(__file__))

if __name__ == '__main__':
	parser = argparse.ArgumentParser(description=__doc__)
	parser.add_argument('homewatcherConfig', help='use HWCONF as the source configuration to resolve.', metavar='HWCONF')
	parser.add_argument('-o', '--output-file', dest='outputFile', help='write the resolved configuration to FILE rather than to standard output.', metavar='FILE')
	parser.add_argument('-v', '--verbose', dest='verbosityLevel', help='set verbosity level.', metavar='LEVEL', choices=[l.lower() for l in logger.getLevelsToString()], default='error')
	args = parser.parse_args()

	# Configure logger.
	logger.initLogger(None, args.verbosityLevel.upper())

	# Read configuration.
	config = Configuration.parseFile(args.homewatcherConfig)

	# Generate a resolved XML.
	config.resolve()
	resolvedDom = config.toXml()
	xmlOutput = resolvedDom.toprettyxml().strip()

	if args.outputFile:
		with open(args.outputFile, 'w') as f:
			f.write(xmlOutput)
	else:
		print(xmlOutput)



#!/usr/bin/python3

# Copyright (C) 2012-2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

import sys

if sys.version_info.major < 3:
	print('Homewatcher is designed to work with Python 3 or above. Your current version is ' + sys.version)

try:
	import pyknx
except ImportError:
	print('Could not import package "pyknx". Make sure it is installed before continuing. You can install it from PyPI with "pip3 install pyknx"')
	exit(1)



#!/usr/bin/python3

# Copyright (C) 2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

# Check that pyknx is present as soon as possible.
from homewatcher import ensurepyknx

from pyknx import logger
import subprocess
import os
import threading
import time

class Timer(threading.Thread):
	def __init__(self, sensor, timeout, name, onTimeoutReached, onIterate = None, onTerminated = None):
		threading.Thread.__init__(self, name=name + ' (id={0})'.format(id(self)))
		self.sensor = sensor
		self.timeout = timeout
		self.reset()
		self.isTerminating = False
		self.isTerminated = False # Mainly for unit testing.
		self.onIterate = onIterate
		self.onTimeoutReached = onTimeoutReached
		self.onTerminated = onTerminated

	def run(self):
		try:
			logger.reportDebug('Starting {0}'.format(self))
			while not self.isTerminating:
				if self.onIterate is not None: self.onIterate(self)

				# Check for termination.
				if self.isTerminating:
					return

				# Main loop.
				if not self.isPaused:
					if self.endTime is None: self.extend(starts=True)
					if time.time() > self.endTime:
						# Execute delayed job.
						logger.reportDebug('Timeout reached for {0}.'.format(self))
						if callable(self.onTimeoutReached): self.onTimeoutReached(self)
						break

				time.sleep(0.2)
		finally:
			if self.isTerminating:
				# Timer has been stopped from outside.
				logger.reportDebug('{0} is canceled.'.format(self))
			else:
				# Maybe useless but set it for consistency.
				self.isTerminating = True
			if callable(self.onTerminated): self.onTerminated(self)
			logger.reportDebug('{0} is now terminated.'.format(self))
			self.isTerminated = True

	def forceTimeout(self):
		logger.reportDebug('Forcing timeout of {0}'.format(self))
		self.endTime = 0

	def pause(self):
		self.isPaused = True

	def stop(self):
		if not self.isTerminating:
			self.isTerminating = True
			logger.reportDebug('Stopping {0}.'.format(self))

	def reset(self):
		self.endTime = None
		self.isPaused = False

	def __str__(self):
		return '{0} => {1} id={2}'.format(self.sensor, self.name, id(self))

	def extend(self, starts = False):
		""" Prolong duration by the timeout amount of time. """
		if self.isTerminating:
			raise Exception('Timer {0} is terminating, it cannot be extended.'.format(self))
		self.endTime = time.time() + self.timeout
		if starts:
			logger.reportDebug('{0} started for {1} seconds.'.format(self, self.timeout))
		else:
			logger.reportDebug('{0} is extended by {1} seconds.'.format(self, self.timeout))

	# class AlertTimer(Timer):
		# def __init__(self, sensor):
			# Sensor.Timer.__init__(self, sensor, sensor.getPrealertTimeout(), 'Alert timer for {0}'.format(sensor.name))
			# self.state = 'prealert'
# 
		# def onTimeoutReached(self):
			# # To be on the safe side, check that this sensor is still the
			# # current one in its related sensor.
			# if self.sensor._alertTimer != self:
				# logger.reportError('{0} is not the current alert timer of {1} (which is currently {2}). onTimeoutReached is aborted and timer stopped.'.format(self, self.sensor, self.sensor._alertTimer))
				# self.stop()
				# return
# 
			# logger.reportDebug('AlertTimer.onTimeoutReached state={0}'.format(self.state))
			# if self.sensor.isInhibited: self.stop()
# 
			# if self.state == 'prealert':
				# logger.reportDebug('AlertTimer.onTimeoutReached isAlertActive={0}'.format(self.sensor.isAlertActive))
				# self.sensor.isAlertActive = True
				# self.state = 'postalert'
				# nextTimeout = self.sensor.getPostalertTimeout()
			# elif self.state == 'postalert':
				# logger.reportDebug('AlertTimer.onTimeoutReached isAlertActive={0}'.format(self.sensor.isAlertActive))
				# self.sensor.isAlertActive = False
				# nextTimeout = None
			# else:
				# raise Exception('Invalid alert timer state.')
# 
			# return nextTimeout
# 
		# def onTerminated(self):
			# self.sensor.isAlertActive = False
# 
		# # def onIterate(self):
			# # if self.state == 'prealert' and self.sensor.hasSimilarAlertAlreadyBeenTriggered:
				# # # Another sensor fired alert, do not wait until our prealert
				# # # delay is over, participate now!
				# # self.forceTimeout()
# 
	# class DelayedActivationTimer(Timer):
		# def __init__(self, sensor):
			# Sensor.Timer.__init__(self, sensor, sensor.getDelayedActivationTimeout(), 'Activation timer for {0}'.format(sensor.name))
			# self.isReady = None
# 
		# def reset(self):
			# Sensor.Timer.reset(self)
			# self.isReady = None
# 
		# def onTimeoutReached(self):
			# # To be on the safe side, check that this sensor is still the
			# # current one in its related sensor.
			# if self.sensor._activationTimer != self:
				# logger.reportError('{0} is not the current activation timer of {1} (which is currently {2}). onTimeoutReached is aborted and timer stopped.'.format(self, self.sensor, self.sensor._activationTimer))
				# self.stop()
				# return
# 
			# self.sensor.isEnabled = True
# 
		# # def onIterate(self):
			# # if not self.sensor.isRequiredByCurrentMode():
				# # self.stop()
			# # if self.isReady is None or not self.isReady:
				# # isFirstPass = self.isReady is None
				# # self.isReady = self.sensor.canBeEnabled()
				# # self.isPaused = not self.isReady
				# # if isFirstPass and not self.isReady:
					# # logger.reportInfo('{0}\'s activation is postponed until {1} returns True.'.format(self.sensor.name, self.sensor.canBeEnabled))
				# # elif self.isReady:
					# # logger.reportInfo('{0} will be activated in {1} seconds.'.format(self.sensor.name, self.timeout))
			# # else:
				# # if not self.sensor.canBeEnabled():
					# # self.reset()
# 



#!/usr/bin/python3

# Copyright (C) 2012-2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

# Check that pyknx is present as soon as possible.
from homewatcher import ensurepyknx

from xml.dom.minidom import parse
from pyknx import logger
import pyknx.configurator
from homewatcher import configuration
import sys
import getopt
import codecs
import logging

class Configurator(pyknx.configurator.Configurator):
	""" Object able to automatically patch the linknx configuration xml to add python callbacks. """
	def __init__(self, homewatcherConfig, sourceFile, outputFile):
		if homewatcherConfig is None:
			self._homewatcherConfig = configuration.Configuration.parseString(self.readFileFromStdIn())
		elif isinstance(homewatcherConfig, str):
			self._homewatcherConfig = configuration.Configuration.parseFile(homewatcherConfig)
		elif isinstance(homewatcherConfig, configuration.Configuration):
			self._homewatcherConfig = homewatcherConfig
		else:
			raise Exception('Unexpected type of object for configuration {0}. Expected a configuration object, a path to a file or None to read from standard input.'.format(homewatcherConfig))
		self._homewatcherConfig.resolve()
		pyknx.configurator.Configurator.__init__(self, sourceFile, outputFile, (self._homewatcherConfig.servicesRepository.daemon.host, self._homewatcherConfig.servicesRepository.daemon.port), 'homewatcher')

	def addCallbackForObject(self, objectId, callbackName, callbackDestination):
		if objectId == None or objectId == '':
			logger.reportWarning('{0} is not defined, skipping callback.'.format(callbackDestination))
			return

		# Search object in config.
		found = False
		for objectXmlConfig in self.config.getElementsByTagName('object'):
			if objectXmlConfig.getAttribute('id') == objectId:
				if found:
					raise Exception('Two objects with id {id} found.'.format(id=objectId))
				found = True
				objectXmlConfig.setAttribute('pyknxcallback', callbackName)
				logger.reportInfo('Added callback {0} for {1}'.format(callbackName, objectId))
		if not found:
			raise Exception('Object {id} not found in linknx configuration'.format(id=objectId))

	def cleanConfig(self):
		for objectXmlConfig in self.config.getElementsByTagName('object'):
			if objectXmlConfig.hasAttribute('pyknxcallback'):
				logger.reportInfo('Removed callback {0} for {1}'.format(objectXmlConfig.getAttribute('pyknxcallback'), objectXmlConfig.getAttribute('id')))
				objectXmlConfig.removeAttribute('pyknxcallback')
		pyknx.configurator.Configurator.cleanConfig(self)

	def generateConfig(self):
		# Add callback for sensors.
		for sensor in self._homewatcherConfig.sensors:
			self.addCallbackForObject(sensor.watchedObjectId, 'onWatchedObjectChanged', 'Watched object for {0}'.format(sensor))

		# Add callbacks for alerts.
		for alert in self._homewatcherConfig.alerts:
			self.addCallbackForObject(alert.persistenceObjectId, 'onAlertPersistenceObjectChanged', 'Persistence for {0}'.format(alert))
			self.addCallbackForObject(alert.inhibitionObjectId, 'onAlertInhibitionObjectChanged', 'Inhibition for {0}'.format(alert))

		# Add callbacks for modes.
		self.addCallbackForObject(self._homewatcherConfig.modesRepository.objectId, 'onModeObjectChanged', 'Mode object')

		pyknx.configurator.Configurator.generateConfig(self)



#!/usr/bin/python3

# Copyright (C) 2012-2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

from distutils.core import setup
import pip
import sys

def installRequirement(requirementName):
	print('************************* IMPORTANT ******************************'.format(requirementName))
	print('The {0} package is required and is not installed on this machine.'.format(requirementName))
	print('You should install {0} first using for instance "pip3 install {0}"'.format(requirementName))
	print('******************************************************************'.format(requirementName))
	# print('Do you want to abort installation of homewatcher in order to complete this required step? (yes/no)')
	# answer = None
	# while not answer:
		# answer = sys.stdin.readline().lower().strip()
		# if not answer in ('yes', 'no'):
			# print('Please answer either yes or no.')
			# answer = None
	# if answer == 'yes':
		# exit(2)
	# args = ['install']
	# if canGetPreVersion: args.append('--pre')
	# args.append(requirementName)
	# pip.main(args)

if sys.version_info.major < 3:
	print('This package is compatible with Python 3 and above.')
	exit(4)

# Install pyknx if required.
try:
	import pyknx
except ImportError:
	installRequirement('pyknx')

# # Install lxml.
# try:
	# import lxml
# except ImportError:
	# installRequirement('lxml')


setup(	name='homewatcher',
		version='0.0.1b10',
		description='Alarm system built on top of Linknx',
		long_description=''.join(open('README.md').readlines()),
		author='Cyrille Defranoux',
		author_email='knx@aminate.net',
		maintainer='Cyrille Defranoux',
		maintainer_email='knx@aminate.net',
		license='GNU Public General License',
		url='https://github.com/2franix/homewatcher/',
		requires=['pyknx (>=2.0)', 'lxml'],
		packages=['homewatcher'],
		data_files=[('.', ['README.md'])],
		scripts=['hwconf.py', 'hwdaemon.py', 'hwresolve.py'])
